/***************************************************************
 * Name:      PE5004_GUI.h
 * Purpose:   Defines Application Frame
 * Author:    jvc ()
 * Created:   2010-11-15
 * Copyright: jvc ()
 * License:
 **************************************************************/

#ifndef PE5004_GUI_H
#define PE5004_GUI_H


#include "5004_main.h"
#include "wx/timer.h"
#include "wx/app.h"
#include "wx/dc.h"
#include "wx/wx.h"
#include "wx/dcbuffer.h"

#include "5004_panel.h"
#include "usb.h"

#define VENDOR_ID 0x102A;
#define PRODUCT_ID 0x5004;
#define VERSION_NB 0x0001;

// USB and PE5004 organisation

#define USB_OFF       0x01
#define USB_ON        0x02
#define PE5004_FIRST  0x01 // board sensors
#define PE5004_SECOND 0x02 // external user defined sensors

class bargraph;
class valuetable;
class valuegrid;
class touch;
class draw;



class calibpanel: public frame_calib
{
    public:
        calibpanel(wxFrame *frame);
        ~calibpanel();


        BYTE HID_IN8[USB_WRBYTELENGTH];
        BYTE HID_OUT8[3][USB_BYTELENGTH];
        BYTE HID_OUT8b[USB_BYTELENGTH];
        BYTE HID_OUT8c[USB_BYTELENGTH];

        BYTE *pHID_OUT8H;
        BYTE USB_read[5];
        int i, runter;
        int Value_COLSTART;
        int Value_ROWSTART;
        int Value_COLEND;
        int Value_ROWEND;
        int Value_WaitCOL;
        int Value_WaitROW;
        int Value_PWRI;
        int Value_PWRP;
        int Value_PWRS;
        int Value_SINE_PUSH;
        int Value_SINF;
        int Value_SINA;
        int Value_PCO;
        int Value_DACREF;
        int Value_GCO;
        int Value_updaterate;
        int Value_DEBMUX;
        int Value_ADC[11][11];
        int empty[11][11];
        int Value_adreal[10][10];
        int Value_admid[11][11];
        int rowthresh[10];
        int Value_adcal[11][11];
        int Value_Continous;
        int Value_SendData;
        int Value_SendRegister;
        int Value_ReadRegister;
        int thresh;
        int alt;
        int stretch;
        int filt;
        int Selected_PE5004;
        BYTE pUSB_write[27];
        bool invert;
        bool boxmode_game;
        bool boxmode_oled;
        bool calibrate;
        int openw;
        bool autoconnect;
        virtual void OnHistory(wxCommandEvent &event);
        virtual void OnExit(wxCommandEvent &event);
        virtual void OnAbout(wxCommandEvent &event);
        void SetConnectStatus(bool bConnected);
        wxCommandEvent blankevent;
        int grafzahl;


    private:

        wxString strFileNameGlobal;
        virtual void OnTimer(wxCommandEvent &event);
        virtual void On_OpenBargraph( wxCommandEvent& event );
        virtual void On_OpenValuetable( wxCommandEvent& event );
        virtual void On_OpenValuegrid( wxCommandEvent& event );
        virtual void On_OpenDraw( wxCommandEvent& event );
        virtual void Scroll_calib( wxScrollEvent& event );
        virtual void Text_calib( wxCommandEvent& event );
        virtual void Choice_calib( wxCommandEvent& event );
        virtual void Update_all();
        virtual void On_SingleShot( wxCommandEvent& event );
        virtual void On_Continous( wxCommandEvent& event );
        virtual void On_Stop( wxCommandEvent& event );
        virtual void On_Connect( wxCommandEvent& event );
        //virtual void On_ReadRegister( wxCommandEvent& event );
        virtual void On_Calib( wxCommandEvent& event );
        virtual void On_ClearCalib( wxCommandEvent& event ) ;
        virtual void On_Preset( wxCommandEvent& event ) ;
        virtual void On_Save( wxCommandEvent& event ) ;
        void UPDATE5004(bool start);
        void READ5004();

        bargraph *pbar[10];
        valuetable *pval;
        valuegrid *pgrid;
        draw *pdraw;
        wxTimer *pTimer;
        char sign;
        bool mode, update;
        CUSBDevice *usbDev;

};

class bargraph: public frame_bargraph
{
    public:
        bargraph(wxFrame *frame, int iRow);
        void Update_all(int iCol,int *iRow, int Value_ADC[11][11]);
        ~bargraph();
        virtual void OnClose(wxCloseEvent& event);

};

class valuetable: public frame_valuetable
{
    public:
        valuetable(wxFrame *frame);
        void Update_all(int *iCol,int *iRow,int *iCols,int *iRows, int Value_ADC[11][11]);
        ~valuetable();
        virtual void OnClose(wxCloseEvent& event);

};

class valuegrid: public frame_valuegrid
{


    public:
        valuegrid(wxFrame *frame);
        void Update_all(int *iCol,int *iRow,int *iCols,int *iRows, int Value_ADC[11][11]);
        char ichar, achar;
        ~valuegrid();
        virtual void OnClose(wxCloseEvent& event);

};

class draw : public frame_draw
{
	public:
	   bool firsttouch;
	   int vec_x_filt;
       int vec_y_filt;


    draw(wxFrame *frame);
	void Update_all(int *iCole,int *iRowe,int *iCols,int *iRows, int Value_ADC[11][11]);
	void Clear();
	~draw();

	virtual void OnClose( wxCloseEvent& event );

};




#endif // PE5004_GUI_H
