
//#include "stdafx.h"
#include "usb.h"
#include "setupapi.h"

extern "C" {
#include "hidsdi.h"
}



CUSBDevice::CUSBDevice()
{
	handle = INVALID_HANDLE_VALUE;
	EventObject = INVALID_HANDLE_VALUE;
	IsConnected = false;

    //HIDOverlapped;  whatever is in there should be fine.
}

CUSBDevice::~CUSBDevice()
{
	Disconnect();
}

CUSBDevice::CUSBDevice(const DWORD *vendorID, const DWORD *productID, const DWORD *versionNumber)
{
	handle = INVALID_HANDLE_VALUE;
	EventObject = INVALID_HANDLE_VALUE;
	//HIDOverlapped;  whatever is in there should be fine.
	Connect(vendorID, productID, versionNumber);
}


HANDLE CUSBDevice::ConnectToIthDevice (DWORD deviceIndex)
{
    GUID hidGUID;
  // GUID *phidGUID;
    HDEVINFO hardwareDeviceInfoSet;
    SP_DEVICE_INTERFACE_DATA deviceInterfaceData;
    PSP_INTERFACE_DEVICE_DETAIL_DATA deviceDetail;
    ULONG requiredSize;
    HANDLE deviceHandle = INVALID_HANDLE_VALUE;
    //HANDLE deviceHandle = 0;
    DWORD result;
    DWORD error_var;
//    phidGUID = new GUID();


    //Get the HID GUID value - used as mask to get list of devices
//    HidD_GetHidGuid (phidGUID);
    HidD_GetHidGuid (&hidGUID);

    //Get a list of devices matching the criteria (hid interface, present)
    hardwareDeviceInfoSet = SetupDiGetClassDevs (&hidGUID,
                                                 NULL, // Define no enumerator (global)
                                                 NULL, // Define no
                                                 //DIGCF_ALLCLASSES );
                                                (DIGCF_PRESENT | // Only Devices present
                                                DIGCF_DEVICEINTERFACE)); // Function class devices.

    deviceInterfaceData.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);

    //Go through the list and get the interface data
    result = SetupDiEnumDeviceInterfaces (hardwareDeviceInfoSet,
                                          NULL, //infoData,
                                          &hidGUID, //interfaceClassGuid,
                                          deviceIndex,
                                          &deviceInterfaceData);

    /* Failed to get a device - possibly the index is larger than the number of devices */
    if (result == FALSE)
    {
        SetupDiDestroyDeviceInfoList (hardwareDeviceInfoSet);
        return INVALID_HANDLE_VALUE;
    }

    //Get the details with null values to get the required size of the buffer
    SetupDiGetDeviceInterfaceDetail (hardwareDeviceInfoSet,
                                     &deviceInterfaceData,
                                     NULL, //interfaceDetail,
                                     0, //interfaceDetailSize,
                                     &requiredSize,
                                     0); //infoData))

    //Allocate the buffer
    deviceDetail = (PSP_INTERFACE_DEVICE_DETAIL_DATA)malloc(requiredSize);
    deviceDetail->cbSize = sizeof(SP_INTERFACE_DEVICE_DETAIL_DATA);

    //Fill the buffer with the device details
    if (!SetupDiGetDeviceInterfaceDetail (hardwareDeviceInfoSet,
		&deviceInterfaceData,
		deviceDetail,
		requiredSize,
		&requiredSize,
		NULL))
    {
        SetupDiDestroyDeviceInfoList (hardwareDeviceInfoSet);
        free (deviceDetail);
        return INVALID_HANDLE_VALUE;
    }

    //Open file on the device
    deviceHandle = CreateFile (deviceDetail->DevicePath,
		GENERIC_READ | GENERIC_WRITE, //GENERIC_READ |
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL,        // no SECURITY_ATTRIBUTES structure
		OPEN_EXISTING, // No special create flags
		0,
		NULL);       // No template file

    error_var=GetLastError();


    SetupDiDestroyDeviceInfoList (hardwareDeviceInfoSet);
    free (deviceDetail);
    return deviceHandle;
}

bool CUSBDevice::Connect(const DWORD *vendorID, const DWORD *productID, const DWORD *versionNumber)
{
	Disconnect();
	IsConnected = false;
    handle = INVALID_HANDLE_VALUE;
    //handle=0;
    DWORD index = 0;
    DWORD result4 = 0;
    HIDD_ATTRIBUTES deviceAttributes;
    BOOL matched = FALSE;
    //while (!matched && (handle = ConnectToIthDevice (index)) != INVALID_HANDLE_VALUE)  // orig structure not on all PC possible
    while (!matched && (index<100))//((handle = ConnectToIthDevice (index)) != INVALID_HANDLE_VALUE))
    {
        handle = ConnectToIthDevice (index);
        if (handle != INVALID_HANDLE_VALUE){
          if (!HidD_GetAttributes (handle, &deviceAttributes))
              return INVALID_HANDLE_VALUE;

          if ((vendorID == 0 || deviceAttributes.VendorID == *vendorID) &&
              (productID == 0 || deviceAttributes.ProductID == *productID) &&
              (versionNumber == 0 || deviceAttributes.VersionNumber == *versionNumber)) {
			  	  EventObject = CreateEvent(NULL, TRUE, TRUE, "");
				  //Set the members of the overlapped structure.
                  result4 = *vendorID;
                  IsConnected = true;

				  return true; /* matched */
		  }
          CloseHandle (handle); /* not a match - close and try again */
        }
        index++;
    }

    return false;
}


#ifdef _DEBUG
void CUSBDevice::ExploreToDebugWindow()
{
	USHORT buffer[128];
    HANDLE deviceHandle = INVALID_HANDLE_VALUE;
    DWORD index = 0;
    HIDD_ATTRIBUTES deviceAttributes;
    BOOL matched = FALSE;

    while (!matched && (deviceHandle = ConnectToIthDevice (index)) != INVALID_HANDLE_VALUE)
    //while (!matched && (deviceHandle = connectToIthUSBHIDDevice (index)) != INVALID_HANDLE_VALUE)
    {
        if (!HidD_GetAttributes (deviceHandle, &deviceAttributes))
            return;

		HidD_GetProductString(deviceHandle, buffer, sizeof(buffer));

		CString strName;
		for (int x = 0; buffer[x]; x++) {
			strName += (char)buffer[x];
		}


		TRACE("HID Device - Vendor %x, Product %x, Version %x, String \'%s\'\r\n", deviceAttributes.VendorID, deviceAttributes.ProductID, deviceAttributes.VersionNumber, strName);
        CloseHandle (deviceHandle); /* not a match - close and try again */

        index++;
    }

    return;
}

#endif

void CUSBDevice::Disconnect()
{
	if (EventObject != INVALID_HANDLE_VALUE)
		CloseHandle(EventObject);
	EventObject = INVALID_HANDLE_VALUE;

	if (handle != INVALID_HANDLE_VALUE)
		CloseHandle(handle);
	handle = INVALID_HANDLE_VALUE;
	IsConnected = false;
}

int CUSBDevice::Write(const BYTE *bytes, int nBuffLen)
{


	unsigned long BytesWritten;
	WriteFile(handle, (LPCVOID)bytes, nBuffLen, &BytesWritten, NULL);
    return GetLastError();

}

int CUSBDevice::Read(BYTE *bytes, int nBuffLen)
{
    DWORD error_var2;
	unsigned long BytesRead = 0;
	ReadFile(handle, (LPVOID)bytes, nBuffLen, &BytesRead, NULL);
	return GetLastError();

}




// DWORD error_var3;


//    HIDP_CAPS                  caps;
//    PHIDP_PREPARSED_DATA       PreparsedData;
//    HIDP_REPORT_TYPE           HidP_Input, HidP_Output;
//    ULONG ValueCapsLength;
//
//HIDP_VALUE_CAPS* pValueCaps = new HIDP_VALUE_CAPS[ValueCapsLength];
//
//HidD_GetPreparsedData(handle,&PreparsedData);
//HidP_GetCaps(PreparsedData, &caps);    // fllen
//ValueCapsLength = caps.NumberOutputValueCaps;
//
//
//
//
//    wxLogDebug("Input - GetValueCaps:\n\n");
//    HidP_GetValueCaps(HidP_Output,
//                      pValueCaps,
//                      &ValueCapsLength,
//                      PreparsedData);
//    delete [] pValueCaps;
//    wxLogDebug("%s0x%X\n", "Input - ReportID: ", caps.OutputReportByteLength);
//




